@@ -915,6 +915,20 @@ int rados_ioctx_snap_remove(rados_ioctx_t io, const char *snapname);
  * @param snapname which snapshot to rollback to
  * @returns 0 on success, negative error code on failure
  */
+int rados_ioctx_snap_rollback(rados_ioctx_t io, const char *oid,
+		   const char *snapname);
+
+/**
+ * Rollback an object to a pool snapshot *DEPRECATED*
+ *
+ * Deprecated interface which is not rados_ioctx_snap_rollback()
+ * This function could go away in the future
+ *
+ * @param io the pool in which the object is stored
+ * @param oid the name of the object to rollback
+ * @param snapname which snapshot to rollback to
+ * @returns 0 on success, negative error code on failure
+ */
 int rados_rollback(rados_ioctx_t io, const char *oid,
 		   const char *snapname);
 
@@ -1064,8 +1078,7 @@ uint64_t rados_get_last_version(rados_ioctx_t io);
  * @param buf data to write
  * @param len length of the data, in bytes
  * @param off byte offset in the object to begin writing at
- * @returns number of bytes written on success, negative error code on
- * failure
+ * @returns 0 on success, negative error code on failure
  */
 int rados_write(rados_ioctx_t io, const char *oid, const char *buf, size_t len, uint64_t off);
 
@@ -1111,8 +1124,7 @@ int rados_clone_range(rados_ioctx_t io, const char *dst, uint64_t dst_off,
  * @param oid the name of the object
  * @param buf the data to append
  * @param len length of buf (in bytes)
- * @returns number of bytes written on success, negative error code on
- * failure
+ * @returns 0 on success, negative error code on failure
  */
 int rados_append(rados_ioctx_t io, const char *oid, const char *buf, size_t len);
 
@@ -1864,7 +1876,9 @@ void rados_write_op_set_flags(rados_write_op_t write_op, int flags);
 void rados_write_op_assert_exists(rados_write_op_t write_op);
 
 /**
- * Ensure that given xattr satisfies comparison
+ * Ensure that given xattr satisfies comparison.
+ * If the comparison is not satisfied, the return code of the
+ * operation will be -ECANCELED
  * @param write_op operation to add this action to
  * @param name name of the xattr to look up
  * @param comparison_operator currently undocumented, look for
@@ -2047,10 +2061,10 @@ void rados_write_op_set_alloc_hint(rados_write_op_t write_op,
 /**
  * Perform a write operation synchronously
  * @param write_op operation to perform
- * @io the ioctx that the object is in
- * @oid the object id
- * @mtime the time to set the mtime to, NULL for the current time
- * @flags flags to apply to the entire operation (LIBRADOS_OPERATION_*)
+ * @param io the ioctx that the object is in
+ * @param oid the object id
+ * @param mtime the time to set the mtime to, NULL for the current time
+ * @param flags flags to apply to the entire operation (LIBRADOS_OPERATION_*)
  */
 int rados_write_op_operate(rados_write_op_t write_op,
 			   rados_ioctx_t io,
@@ -2060,11 +2074,11 @@ int rados_write_op_operate(rados_write_op_t write_op,
 /**
  * Perform a write operation asynchronously
  * @param write_op operation to perform
- * @io the ioctx that the object is in
+ * @param io the ioctx that the object is in
  * @param completion what to do when operation has been attempted
- * @oid the object id
- * @mtime the time to set the mtime to, NULL for the current time
- * @flags flags to apply to the entire operation (LIBRADOS_OPERATION_*)
+ * @param oid the object id
+ * @param mtime the time to set the mtime to, NULL for the current time
+ * @param flags flags to apply to the entire operation (LIBRADOS_OPERATION_*)
  */
 int rados_aio_write_op_operate(rados_write_op_t write_op,
                                rados_ioctx_t io,
@@ -2104,6 +2118,8 @@ void rados_read_op_assert_exists(rados_read_op_t read_op);
 
 /**
  * Ensure that the an xattr satisfies a comparison
+ * If the comparison is not satisfied, the return code of the
+ * operation will be -ECANCELED
  * @param read_op operation to add this action to
  * @param name name of the xattr to look up
  * @param comparison_operator currently undocumented, look for
@@ -2290,7 +2306,7 @@ void rados_read_op_omap_get_vals_by_keys(rados_read_op_t read_op,
 					 int *prval);
 
 /**
- * Perform a write operation synchronously
+ * Perform a read operation synchronously
  * @param read_op operation to perform
  * @io the ioctx that the object is in
  * @oid the object id
@@ -2302,7 +2318,7 @@ int rados_read_op_operate(rados_read_op_t read_op,
 			  int flags);
 
 /**
- * Perform a write operation asynchronously
+ * Perform a read operation asynchronously
  * @param read_op operation to perform
  * @io the ioctx that the object is in
  * @param completion what to do when operation has been attempted
