@@ -131,8 +131,11 @@ int MonClient::get_monmap_privately()
   while (monmap.fsid.is_zero()) {
     cur_mon = _pick_random_mon();
     cur_con = messenger->get_connection(monmap.get_inst(cur_mon));
-    ldout(cct, 10) << "querying mon." << cur_mon << " " << cur_con->get_peer_addr() << dendl;
-    messenger->send_message(new MMonGetMap, cur_con);
+    if (cur_con) {
+      ldout(cct, 10) << "querying mon." << cur_mon << " "
+		     << cur_con->get_peer_addr() << dendl;
+      cur_con->send_message(new MMonGetMap);
+    }
 
     if (--attempt == 0)
       break;
@@ -141,14 +144,16 @@ int MonClient::get_monmap_privately()
     interval.set_from_double(cct->_conf->mon_client_hunt_interval);
     map_cond.WaitInterval(cct, monc_lock, interval);
 
-    if (monmap.fsid.is_zero()) {
-      messenger->mark_down(cur_con);  // nope, clean that connection up
+    if (monmap.fsid.is_zero() && cur_con) {
+      cur_con->mark_down();  // nope, clean that connection up
     }
   }
 
   if (temp_msgr) {
-    messenger->mark_down(cur_con);
-    cur_con.reset(NULL);
+    if (cur_con) {
+      cur_con->mark_down();
+      cur_con.reset(NULL);
+    }
     monc_lock.Unlock();
     messenger->shutdown();
     if (smessenger)
@@ -222,7 +227,7 @@ int MonClient::ping_monitor(const string &mon_id, string *result_reply)
   ConnectionRef con = smsgr->get_connection(monmap.get_inst(mon_id));
   ldout(cct, 10) << __func__ << " ping mon." << mon_id
                  << " " << con->get_peer_addr() << dendl;
-  smsgr->send_message(new MPing, con);
+  con->send_message(new MPing);
 
   pinger->lock.Lock();
   int ret = pinger->wait_for_reply(cct->_conf->client_mount_timeout);
@@ -233,7 +238,7 @@ int MonClient::ping_monitor(const string &mon_id, string *result_reply)
   }
   pinger->lock.Unlock();
 
-  smsgr->mark_down(con);
+  con->mark_down();
   smsgr->shutdown();
   smsgr->wait();
   delete smsgr;
@@ -327,8 +332,6 @@ void MonClient::handle_monmap(MMonMap *m)
   if (!monmap.get_addr_name(cur_con->get_peer_addr(), cur_mon)) {
     ldout(cct, 10) << "mon." << cur_mon << " went away" << dendl;
     _reopen_session();  // can't find the mon we were talking to (above)
-  } else {
-    _finish_hunting();
   }
 
   map_cond.Signal();
@@ -418,7 +421,8 @@ void MonClient::shutdown()
   monc_lock.Lock();
   timer.shutdown();
 
-  messenger->mark_down(cur_con);
+  if (cur_con)
+    cur_con->mark_down();
   cur_con.reset(NULL);
 
   monc_lock.Unlock();
@@ -429,7 +433,7 @@ int MonClient::authenticate(double timeout)
   Mutex::Locker lock(monc_lock);
 
   if (state == MC_STATE_HAVE_SESSION) {
-    ldout(cct, 5) << "already authenticated" << dendl;;
+    ldout(cct, 5) << "already authenticated" << dendl;
     return 0;
   }
 
@@ -504,6 +508,7 @@ void MonClient::handle_auth(MAuthReply *m)
   if (ret == -EAGAIN) {
     MAuth *ma = new MAuth;
     ma->protocol = auth->get_protocol();
+    auth->prepare_build_request();
     ret = auth->build_request(ma->auth_payload);
     _send_mon_message(ma, true);
     return;
@@ -553,7 +558,7 @@ void MonClient::_send_mon_message(Message *m, bool force)
     assert(cur_con);
     ldout(cct, 10) << "_send_mon_message to mon." << cur_mon
 		   << " at " << cur_con->get_peer_addr() << dendl;
-    messenger->send_message(m, cur_con);
+    cur_con->send_message(m);
   } else {
     waiting_for_session.push_back(m);
   }
@@ -594,7 +599,7 @@ void MonClient::_reopen_session(int rank, string name)
   }
 
   if (cur_con) {
-    messenger->mark_down(cur_con);
+    cur_con->mark_down();
   }
   cur_con = messenger->get_connection(monmap.get_inst(cur_mon));
 	
@@ -631,7 +636,7 @@ void MonClient::_reopen_session(int rank, string name)
   // send an initial keepalive to ensure our timestamp is valid by the
   // time we are in an OPENED state (by sequencing this before
   // authentication).
-  messenger->send_keepalive(cur_con.get());
+  cur_con->send_keepalive();
 
   MAuth *m = new MAuth;
   m->protocol = 0;
@@ -700,7 +705,7 @@ void MonClient::tick()
     if (now > sub_renew_after)
       _renew_subs();
 
-    messenger->send_keepalive(cur_con.get());
+    cur_con->send_keepalive();
 
     if (state == MC_STATE_HAVE_SESSION) {
       send_log();
@@ -718,9 +723,6 @@ void MonClient::tick()
     }
   }
 
-  if (auth)
-    auth->tick();
-
   schedule_tick();
 }
 
@@ -758,8 +760,6 @@ void MonClient::_renew_subs()
 
 void MonClient::handle_subscribe_ack(MMonSubscribeAck *m)
 {
-  _finish_hunting();
-
   if (sub_renew_sent != utime_t()) {
     sub_renew_after = sub_renew_sent;
     sub_renew_after += m->interval / 2.0;
@@ -780,6 +780,7 @@ int MonClient::_check_auth_tickets()
       ldout(cct, 10) << "_check_auth_tickets getting new tickets!" << dendl;
       MAuth *m = new MAuth;
       m->protocol = auth->get_protocol();
+      auth->prepare_build_request();
       auth->build_request(m->auth_payload);
       _send_mon_message(m);
     }
