@@ -103,14 +103,12 @@ int FileJournal::_open(bool forwrite, bool create)
     goto out_fd;
 
 #ifdef HAVE_LIBAIO
-  if (aio) {
-    aio_ctx = 0;
-    ret = io_setup(128, &aio_ctx);
-    if (ret < 0) {
-      ret = errno;
-      derr << "FileJournal::_open: unable to setup io_context " << cpp_strerror(ret) << dendl;
-      goto out_fd;
-    }
+  aio_ctx = 0;
+  ret = io_setup(128, &aio_ctx);
+  if (ret < 0) {
+    ret = errno;
+    derr << "FileJournal::_open: unable to setup io_context " << cpp_strerror(ret) << dendl;
+    goto out_fd;
   }
 #endif
 
@@ -546,7 +544,6 @@ void FileJournal::close()
 
   // close
   assert(writeq_empty());
-  assert(!must_write_header);
   assert(fd >= 0);
   VOID_TEMP_FAILURE_RETRY(::close(fd));
   fd = -1;
@@ -607,8 +604,7 @@ void FileJournal::start_writer()
   write_stop = false;
   write_thread.create();
 #ifdef HAVE_LIBAIO
-  if (aio)
-    write_finish_thread.create();
+  write_finish_thread.create();
 #endif
 }
 
@@ -617,25 +613,19 @@ void FileJournal::stop_writer()
   {
     Mutex::Locker l(write_lock);
 #ifdef HAVE_LIBAIO
-    if (aio)
-      aio_lock.Lock();
+    Mutex::Locker q(aio_lock);
 #endif
     Mutex::Locker p(writeq_lock);
     write_stop = true;
     writeq_cond.Signal();
 #ifdef HAVE_LIBAIO
-    if (aio) {
-      aio_cond.Signal();
-      write_finish_cond.Signal();
-      aio_lock.Unlock();
-    }
+    aio_cond.Signal();
+    write_finish_cond.Signal();
 #endif
   } 
   write_thread.join();
 #ifdef HAVE_LIBAIO
-  if (aio) {
-    write_finish_thread.join();
-  }
+  write_finish_thread.join();
 #endif
 }
 
@@ -659,13 +649,6 @@ int FileJournal::read_header()
   buffer::ptr bp = buffer::create_page_aligned(block_size);
   bp.zero();
   int r = ::pread(fd, bp.c_str(), bp.length(), 0);
-
-  if (r < 0) {
-    int err = errno;
-    dout(0) << "read_header got " << cpp_strerror(err) << dendl;
-    return -err;
-  }
-
   bl.push_back(bp);
 
   try {
@@ -677,6 +660,11 @@ int FileJournal::read_header()
     return -EINVAL;
   }
 
+  if (r < 0) {
+    int err = errno;
+    dout(0) << "read_header got " << cpp_strerror(err) << dendl;
+    return -err;
+  }
   
   /*
    * Unfortunately we weren't initializing the flags field for new
@@ -806,8 +794,7 @@ int FileJournal::prepare_multi_write(bufferlist& bl, uint64_t& orig_ops, uint64_
   }
 
   dout(20) << "prepare_multi_write queue_pos now " << queue_pos << dendl;
-  assert((write_pos + bl.length() == queue_pos) ||
-         (write_pos + bl.length() - header.max_size + get_top() == queue_pos));
+  //assert(write_pos + bl.length() == queue_pos);
   return 0;
 }
 
@@ -1006,32 +993,22 @@ void FileJournal::do_write(bufferlist& bl)
     dout(10) << "do_write wrapping, first bit at " << pos << " len " << first.length()
 	     << " second bit len " << second.length() << " (orig len " << bl.length() << ")" << dendl;
 
-    //Save pos to write first piece second
-    off64_t first_pos = pos;
-    off64_t orig_pos;
-    pos = get_top();
-    // header too?
+    if (write_bl(pos, first)) {
+      derr << "FileJournal::do_write: write_bl(pos=" << pos
+	   << ") failed" << dendl;
+      ceph_abort();
+    }
+    assert(pos == get_top());
     if (hbp.length()) {
       // be sneaky: include the header in the second fragment
       second.push_front(hbp);
       pos = 0;          // we included the header
     }
-    // Write the second portion first possible with the header, so
-    // do_read_entry() won't even get a valid entry_header_t if there
-    // is a crash between the two writes.
-    orig_pos = pos;
     if (write_bl(pos, second)) {
-      derr << "FileJournal::do_write: write_bl(pos=" << orig_pos
+      derr << "FileJournal::do_write: write_bl(pos=" << pos
 	   << ") failed" << dendl;
       ceph_abort();
     }
-    orig_pos = first_pos;
-    if (write_bl(first_pos, first)) {
-      derr << "FileJournal::do_write: write_bl(pos=" << orig_pos
-	   << ") failed" << dendl;
-      ceph_abort();
-    }
-    assert(first_pos == get_top());
   } else {
     // header too?
     if (hbp.length()) {
@@ -1125,7 +1102,7 @@ void FileJournal::write_thread_entry()
   while (1) {
     {
       Mutex::Locker locker(writeq_lock);
-      if (writeq.empty() && !must_write_header) {
+      if (writeq.empty()) {
 	if (write_stop)
 	  break;
 	dout(20) << "write_thread_entry going to sleep" << dendl;
@@ -1136,9 +1113,7 @@ void FileJournal::write_thread_entry()
     }
     
 #ifdef HAVE_LIBAIO
-    //We hope write_finish_thread_entry return until the last aios complete
-    //when set write_stop. But it can't. So don't use aio mode when shutdown.
-    if (aio && !write_stop) {
+    if (aio) {
       Mutex::Locker locker(aio_lock);
       // should we back off to limit aios in flight?  try to do this
       // adaptively so that we submit larger aios once we have lots of
@@ -1189,7 +1164,7 @@ void FileJournal::write_thread_entry()
     }
 
 #ifdef HAVE_LIBAIO
-    if (aio && !write_stop)
+    if (aio)
       do_aio_write(bl);
     else
       do_write(bl);
@@ -1378,7 +1353,7 @@ void FileJournal::write_finish_thread_entry()
 	aio_info *ai = (aio_info *)event[i].obj;
 	if (event[i].res != ai->len) {
 	  derr << "aio to " << ai->off << "~" << ai->len
-	       << " wrote " << event[i].res << dendl;
+	       << " got " << cpp_strerror(event[i].res) << dendl;
 	  assert(0 == "unexpected aio error");
 	}
 	dout(10) << "write_finish_thread_entry aio " << ai->off
@@ -1401,7 +1376,7 @@ void FileJournal::check_aio_completion()
   assert(aio_lock.is_locked());
   dout(20) << "check_aio_completion" << dendl;
 
-  bool completed_something = false, signal = false;
+  bool completed_something = false;
   uint64_t new_journaled_seq = 0;
 
   list<aio_info>::iterator p = aio_queue.begin();
@@ -1415,7 +1390,6 @@ void FileJournal::check_aio_completion()
     aio_num--;
     aio_bytes -= p->len;
     aio_queue.erase(p++);
-    signal = true;
   }
 
   if (completed_something) {
@@ -1435,8 +1409,7 @@ void FileJournal::check_aio_completion()
 	queue_completions_thru(journaled_seq);
       }
     }
-  }
-  if (signal) {
+
     // maybe write queue was waiting for aio count to drop?
     aio_cond.Signal();
   }
@@ -1712,14 +1685,13 @@ bool FileJournal::read_entry(
 }
 
 FileJournal::read_entry_result FileJournal::do_read_entry(
-  off64_t init_pos,
+  off64_t pos,
   off64_t *next_pos,
   bufferlist *bl,
   uint64_t *seq,
   ostream *ss,
   entry_header_t *_h)
 {
-  off64_t cur_pos = init_pos;
   bufferlist _bl;
   if (!bl)
     bl = &_bl;
@@ -1728,40 +1700,40 @@ FileJournal::read_entry_result FileJournal::do_read_entry(
   entry_header_t *h;
   bufferlist hbl;
   off64_t _next_pos;
-  wrap_read_bl(cur_pos, sizeof(*h), &hbl, &_next_pos);
+  wrap_read_bl(pos, sizeof(*h), &hbl, &_next_pos);
   h = (entry_header_t *)hbl.c_str();
 
-  if (!h->check_magic(cur_pos, header.get_fsid64())) {
-    dout(25) << "read_entry " << init_pos
+  if (!h->check_magic(pos, header.get_fsid64())) {
+    dout(25) << "read_entry " << pos
 	     << " : bad header magic, end of journal" << dendl;
     if (ss)
       *ss << "bad header magic";
     if (next_pos)
-      *next_pos = init_pos + (4<<10); // check 4k ahead
+      *next_pos = pos + (4<<10); // check 4k ahead
     return MAYBE_CORRUPT;
   }
-  cur_pos = _next_pos;
+  pos = _next_pos;
 
   // pad + body + pad
   if (h->pre_pad)
-    cur_pos += h->pre_pad;
+    pos += h->pre_pad;
 
   bl->clear();
-  wrap_read_bl(cur_pos, h->len, bl, &cur_pos);
+  wrap_read_bl(pos, h->len, bl, &pos);
 
   if (h->post_pad)
-    cur_pos += h->post_pad;
+    pos += h->post_pad;
 
   // footer
   entry_header_t *f;
   bufferlist fbl;
-  wrap_read_bl(cur_pos, sizeof(*f), &fbl, &cur_pos);
+  wrap_read_bl(pos, sizeof(*f), &fbl, &pos);
   f = (entry_header_t *)fbl.c_str();
   if (memcmp(f, h, sizeof(*f))) {
     if (ss)
       *ss << "bad footer magic, partial entry";
     if (next_pos)
-      *next_pos = cur_pos;
+      *next_pos = pos;
     return MAYBE_CORRUPT;
   }
 
@@ -1773,13 +1745,13 @@ FileJournal::read_entry_result FileJournal::do_read_entry(
 	*ss << "header crc (" << h->crc32c
 	    << ") doesn't match body crc (" << actual_crc << ")";
       if (next_pos)
-	*next_pos = cur_pos;
+	*next_pos = pos;
       return MAYBE_CORRUPT;
     }
   }
 
   // yay!
-  dout(2) << "read_entry " << init_pos << " : seq " << h->seq
+  dout(2) << "read_entry " << pos << " : seq " << h->seq
 	  << " " << h->len << " bytes"
 	  << dendl;
 
@@ -1791,15 +1763,15 @@ FileJournal::read_entry_result FileJournal::do_read_entry(
   // bind by reference to (packed) h->seq
   journalq.push_back(
     pair<uint64_t,off64_t>(static_cast<uint64_t>(h->seq),
-			   static_cast<off64_t>(init_pos)));
+			   static_cast<off64_t>(pos)));
 
   if (next_pos)
-    *next_pos = cur_pos;
+    *next_pos = pos;
 
   if (_h)
     *_h = *h;
 
-  assert(cur_pos % header.alignment == 0);
+  assert(pos % header.alignment == 0);
   return SUCCESS;
 }
 
