@@ -39,11 +39,11 @@
  * given by the hex characters in the hash beginning with the least
  * significant.
  * 
- * ex: hobject_t("object", CEPH_NO_SNAP, 0xA4CEE0D2)
+ * ex: ghobject_t("object", CEPH_NO_SNAP, 0xA4CEE0D2)
  * would be located in (root)/2/D/0/
  * 
  * Subdirectories are created when the number of objects in a directory
- * exceed 32*merge_threshhold.  The number of objects in a directory 
+ * exceed (abs(merge_threshhold)) * 16 * split_multiplier.  The number of objects in a directory 
  * is encoded as subdir_info_s in an xattr on the directory.
  */
 class HashIndex : public LFNIndex {
@@ -60,7 +60,8 @@ private:
   /**
    * Merges occur when the number of object drops below
    * merge_threshold and splits occur when the number of objects
-   * exceeds 16 * merge_threshold * split_multiplier.
+   * exceeds 16 * abs(merge_threshold) * split_multiplier.
+   * Please note if merge_threshold is less than zero, it will never do merging
    */
   int merge_threshold;
   int split_multiplier;
@@ -155,7 +156,7 @@ public:
   int _split(
     uint32_t match,
     uint32_t bits,
-    std::tr1::shared_ptr<CollectionIndex> dest
+    CollectionIndex* dest
     );
 	
 protected:
@@ -163,30 +164,30 @@ protected:
 
   int _created(
     const vector<string> &path,
-    const hobject_t &hoid,
+    const ghobject_t &oid,
     const string &mangled_name
     );
   int _remove(
     const vector<string> &path,
-    const hobject_t &hoid,
+    const ghobject_t &oid,
     const string &mangled_name
     );
   int _lookup(
-    const hobject_t &hoid,
+    const ghobject_t &oid,
     vector<string> *path,
     string *mangled_name,
     int *exists
     );
   int _collection_list(
-    vector<hobject_t> *ls
+    vector<ghobject_t> *ls
     );
   int _collection_list_partial(
-    const hobject_t &start,
+    const ghobject_t &start,
     int min_count,
     int max_count,
     snapid_t seq,
-    vector<hobject_t> *ls,
-    hobject_t *next
+    vector<ghobject_t> *ls,
+    ghobject_t *next
     );
 private:
   /// Recursively remove path and its subdirs
@@ -262,7 +263,7 @@ private:
 
   /// Determine path components from hoid hash
   void get_path_components(
-    const hobject_t &hoid, ///< [in] Object for which to get path components
+    const ghobject_t &oid, ///< [in] Object for which to get path components
     vector<string> *path   ///< [out] Path components for hoid.
     );
 
@@ -278,12 +279,12 @@ private:
     
 
   /** 
-   * Get string representation of hobject_t/hash
+   * Get string representation of ghobject_t/hash
    *
    * e.g: 0x01234567 -> "76543210"
    */
   static string get_path_str(
-    const hobject_t &hoid ///< [in] Object to get hash string for
+    const ghobject_t &oid ///< [in] Object to get hash string for
     ); ///< @return Hash string for hoid.
 
   /// Get string from hash, @see get_path_str
@@ -319,20 +320,20 @@ private:
   int get_path_contents_by_hash(
     const vector<string> &path,            /// [in] Path to list
     const string *lower_bound,             /// [in] list > *lower_bound
-    const hobject_t *next_object,          /// [in] list > *next_object
+    const ghobject_t *next_object,          /// [in] list > *next_object
     const snapid_t *seq,                   /// [in] list >= *seq
     set<string> *hash_prefixes,            /// [out] prefixes in dir
-    set<pair<string, hobject_t> > *objects /// [out] objects
+    set<pair<string, ghobject_t> > *objects /// [out] objects
     );
 
-  /// List objects in collection in hobject_t order
+  /// List objects in collection in ghobject_t order
   int list_by_hash(
     const vector<string> &path, /// [in] Path to list
     int min_count,              /// [in] List at least min_count
     int max_count,              /// [in] List at most max_count
     snapid_t seq,               /// [in] list only objects where snap >= seq
-    hobject_t *next,            /// [in,out] List objects >= *next
-    vector<hobject_t> *out      /// [out] Listed objects
+    ghobject_t *next,            /// [in,out] List objects >= *next
+    vector<ghobject_t> *out      /// [out] Listed objects
     ); ///< @return Error Code, 0 on success
 };
 
