@@ -98,8 +98,6 @@ class LFNIndex : public CollectionIndex {
 
   /// Path to Index base.
   const string base_path;
-  /// For reference counting the collection @see Path
-  ceph::weak_ptr<CollectionIndex> self_ref;
 
 protected:
   const uint32_t index_version;
@@ -123,7 +121,7 @@ protected:
   }
 
 private:
-  string lfn_attribute;
+  string lfn_attribute, lfn_alt_attribute;
   coll_t collection;
 
 public:
@@ -133,7 +131,8 @@ public:
     const char *base_path, ///< [in] path to Index root
     uint32_t index_version,
     double _error_injection_probability=0)
-    : base_path(base_path),
+    : CollectionIndex(collection), 
+      base_path(base_path),
       index_version(index_version),
       error_injection_enabled(false),
       error_injection_on(_error_injection_probability != 0),
@@ -146,7 +145,8 @@ public:
       char buf[100];
       snprintf(buf, sizeof(buf), "%d", index_version);
       lfn_attribute = LFN_ATTR + string(buf);
-    }
+      lfn_alt_attribute = LFN_ATTR + string(buf) + "-alt";
+   }
   }
 
   coll_t coll() const { return collection; }
@@ -154,9 +154,6 @@ public:
   /// Virtual destructor
   virtual ~LFNIndex() {}
 
-  /// @see CollectionIndex
-  void set_ref(ceph::shared_ptr<CollectionIndex> ref);
-
   /// @see CollectionIndex
   int init();
 
@@ -199,14 +196,14 @@ public:
   virtual int _split(
     uint32_t match,                             //< [in] value to match
     uint32_t bits,                              //< [in] bits to check
-    ceph::shared_ptr<CollectionIndex> dest  //< [in] destination index
+    CollectionIndex* dest                       //< [in] destination index
     ) = 0;
   
   /// @see CollectionIndex
   int split(
     uint32_t match,
     uint32_t bits,
-    ceph::shared_ptr<CollectionIndex> dest
+    CollectionIndex* dest
     ) {
     WRAP_RETRY(
       r = _split(match, bits, dest);
@@ -423,6 +420,9 @@ private:
   const string &get_lfn_attr() const {
     return lfn_attribute;
   }
+  const string &get_alt_lfn_attr() const {
+    return lfn_alt_attribute;
+  }
 
   /**
    * Gets the filename corresponsing to oid in path.
