@@ -248,7 +248,10 @@ void Objecter::init_unlocked()
 					   "objecter_requests",
 					   m_request_state_hook,
 					   "show in-progress osd requests");
-  if (ret < 0) {
+
+  /* Don't warn on EEXIST, happens if multiple ceph clients
+   * are instantiated from one process */
+  if (ret < 0 && ret != -EEXIST) {
     lderr(cct) << "error registering admin socket command: "
 	       << cpp_strerror(ret) << dendl;
   }
@@ -607,7 +610,7 @@ void Objecter::handle_osd_map(MOSDMap *m)
 	  logger->inc(l_osdc_map_full);
 	}
 	else {
-	  if (e >= m->get_oldest()) {
+	  if (e && e > m->get_oldest()) {
 	    ldout(cct, 3) << "handle_osd_map requesting missing epoch " << osdmap->get_epoch()+1 << dendl;
 	    maybe_request_map();
 	    break;
@@ -944,7 +947,7 @@ void Objecter::reopen_session(OSDSession *s)
   entity_inst_t inst = osdmap->get_inst(s->osd);
   ldout(cct, 10) << "reopen_session osd." << s->osd << " session, addr now " << inst << dendl;
   if (s->con) {
-    messenger->mark_down(s->con);
+    s->con->mark_down();
     logger->inc(l_osdc_osd_session_close);
   }
   s->con = messenger->get_connection(inst);
@@ -956,7 +959,7 @@ void Objecter::close_session(OSDSession *s)
 {
   ldout(cct, 10) << "close_session for osd." << s->osd << dendl;
   if (s->con) {
-    messenger->mark_down(s->con);
+    s->con->mark_down();
     logger->inc(l_osdc_osd_session_close);
   }
   s->ops.clear();
@@ -1153,7 +1156,7 @@ void Objecter::tick()
     for (set<OSDSession*>::iterator i = toping.begin();
 	 i != toping.end();
 	 ++i) {
-      messenger->send_message(new MPing, (*i)->con);
+      (*i)->con->send_message(new MPing);
     }
   }
     
@@ -1364,11 +1367,6 @@ int Objecter::op_cancel(ceph_tid_t tid, int r)
 
   ldout(cct, 10) << __func__ << " tid " << tid << dendl;
   Op *op = p->second;
-  if (op->con) {
-    ldout(cct, 20) << " revoking rx buffer for " << tid
-		   << " on " << op->con << dendl;
-    op->con->revoke_rx_buffer(tid);
-  }
   if (op->onack) {
     op->onack->complete(r);
     op->onack = NULL;
@@ -1439,7 +1437,7 @@ int64_t Objecter::get_object_pg_hash_position(int64_t pool, const string& key,
   return p->raw_hash_to_pg(p->hash_key(key, ns));
 }
 
-int Objecter::calc_target(op_target_t *t, bool any_change)
+int Objecter::calc_target(op_target_t *t)
 {
   bool is_read = t->flags & CEPH_OSD_FLAG_READ;
   bool is_write = t->flags & CEPH_OSD_FLAG_WRITE;
@@ -1496,8 +1494,7 @@ int Objecter::calc_target(op_target_t *t, bool any_change)
   }
 
   if (t->pgid != pgid ||
-      is_pg_changed(
-	t->primary, t->acting, primary, acting, t->used_replica || any_change) ||
+      is_pg_changed(t->primary, t->acting, primary, acting, t->used_replica) ||
       force_resend) {
     t->pgid = pgid;
     t->acting = acting;
@@ -1521,7 +1518,7 @@ int Objecter::calc_target(op_target_t *t, bool any_change)
 	// look for a local replica.  prefer the primary if the
 	// distance is the same.
 	int best = -1;
-	int best_locality;
+	int best_locality = 0;
 	for (unsigned i = 0; i < acting.size(); ++i) {
 	  int locality = osdmap->crush->get_common_ancestor_distance(
 		 cct, acting[i], crush_location);
@@ -1576,7 +1573,7 @@ int Objecter::recalc_op_target(Op *op)
 
 bool Objecter::recalc_linger_op_target(LingerOp *linger_op)
 {
-  int r = calc_target(&linger_op->target, true);
+  int r = calc_target(&linger_op->target);
   if (r == RECALC_OP_TARGET_NEED_RESEND) {
     ldout(cct, 10) << "recalc_linger_op_target tid " << linger_op->linger_id
 		   << " pgid " << linger_op->target.pgid
@@ -1615,10 +1612,7 @@ void Objecter::finish_op(Op *op)
 
   ops.erase(op->tid);
   logger->set(l_osdc_op_active, ops.size());
-
-  // our reply may have raced with pool deletion resulting in a map
-  // check in flight.
-  op_cancel_map_check(op);
+  assert(check_latest_map_ops.find(op->tid) == check_latest_map_ops.end());
 
   if (op->ontimeout)
     timer.cancel_event(op->ontimeout);
@@ -1631,6 +1625,7 @@ void Objecter::send_op(Op *op)
   ldout(cct, 15) << "send_op " << op->tid << " to osd." << op->session->osd << dendl;
 
   int flags = op->target.flags;
+  flags |= CEPH_OSD_FLAG_KNOWN_REDIR;
   if (op->oncommit)
     flags |= CEPH_OSD_FLAG_ONDISK;
   if (op->onack)
@@ -1643,9 +1638,7 @@ void Objecter::send_op(Op *op)
     ldout(cct, 20) << " revoking rx buffer for " << op->tid << " on " << op->con << dendl;
     op->con->revoke_rx_buffer(op->tid);
   }
-  if (op->outbl &&
-      op->ontimeout == NULL &&  // only post rx_buffer if no timeout; see #9582
-      op->outbl->length()) {
+  if (op->outbl && op->outbl->length()) {
     ldout(cct, 20) << " posting rx buffer for " << op->tid << " on " << op->session->con << dendl;
     op->con = op->session->con;
     op->con->post_rx_buffer(op->tid, *op->outbl);
@@ -1680,7 +1673,7 @@ void Objecter::send_op(Op *op)
   logger->inc(l_osdc_op_send);
   logger->inc(l_osdc_op_send_bytes, m->get_data().length());
 
-  messenger->send_message(m, op->session->con);
+  op->session->con->send_message(m);
 }
 
 int Objecter::calc_op_budget(Op *op)
@@ -1779,6 +1772,7 @@ void Objecter::handle_osd_op_reply(MOSDOpReply *m)
     unregister_op(op);
     m->get_redirect().combine_with_locator(op->target.target_oloc,
 					   op->target.target_oid.name);
+    op->target.flags |= CEPH_OSD_FLAG_REDIRECTED;
     _op_submit(op);
     m->put();
     return;
@@ -2862,7 +2856,7 @@ void Objecter::_send_command(CommandOp *c)
   m->cmd = c->cmd;
   m->set_data(c->inbl);
   m->set_tid(c->tid);
-  messenger->send_message(m, c->session->con);
+  c->session->con->send_message(m);
   logger->inc(l_osdc_command_send);
 }
 
