@@ -1,3 +1,5 @@
+// -*- mode:C++; tab-width:8; c-basic-offset:2; indent-tabs-mode:t -*-
+// vim: ts=8 sw=2 smarttab
 
 #include <string.h>
 
@@ -42,7 +44,7 @@ int RGWMongoose::complete_request()
 
       if (0 && data.length() == 0) {
         has_content_length = true;
-        print("Transfer-Enconding: %s\r\n", "chunked");
+        print("Transfer-Enconding: %s\n", "chunked");
         data.append("0\r\n\r\n", sizeof("0\r\n\r\n")-1);
       } else {
         int r = send_content_length(data.length());
@@ -128,7 +130,7 @@ int RGWMongoose::send_status(const char *status, const char *status_name)
   if (!status_name)
     status_name = "";
 
-  snprintf(buf, sizeof(buf), "HTTP/1.1 %s %s\r\n", status, status_name);
+  snprintf(buf, sizeof(buf), "HTTP/1.1 %s %s\n", status, status_name);
 
   bufferlist bl;
   bl.append(buf);
@@ -145,20 +147,6 @@ int RGWMongoose::send_100_continue()
   return mg_write(conn, buf, sizeof(buf) - 1);
 }
 
-static void dump_date_header(bufferlist &out)
-{
-  char timestr[TIME_BUF_SIZE];
-  const time_t gtime = time(NULL);
-  struct tm result;
-  struct tm const * const tmp = gmtime_r(&gtime, &result);
-
-  if (tmp == NULL)
-    return;
-
-  if (strftime(timestr, sizeof(timestr), "Date: %a, %d %b %Y %H:%M:%S %Z\r\n", tmp))
-    out.append(timestr);
-}
-
 int RGWMongoose::complete_header()
 {
   header_done = true;
@@ -167,8 +155,6 @@ int RGWMongoose::complete_header()
     return 0;
   }
 
-  dump_date_header(header_data);
-
   if (explicit_keepalive)
     header_data.append("Connection: Keep-Alive\r\n");
 
@@ -183,6 +169,6 @@ int RGWMongoose::send_content_length(uint64_t len)
 {
   has_content_length = true;
   char buf[21];
-  snprintf(buf, sizeof(buf), "%"PRIu64, len);
-  return print("Content-Length: %s\r\n", buf);
+  snprintf(buf, sizeof(buf), "%" PRIu64, len);
+  return print("Content-Length: %s\n", buf);
 }
